package com.github.polpetta.mezzotre.telegram.command;

import com.github.polpetta.mezzotre.i18n.TemplateContentGenerator;
import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.mezzotre.util.Clock;
import com.github.polpetta.types.json.ChatContext;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.SendMessage;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;
import javax.inject.Inject;
import javax.inject.Named;
import org.apache.commons.lang3.tuple.Pair;

public class Help implements Processor {

  private static final String TRIGGERING_STAGING_NAME = "/help";

  private final TemplateContentGenerator templateContentGenerator;
  private final Executor threadPool;
  private final Clock clock;
  private final Map<String, Processor> tgCommandProcessors;

  @Inject
  public Help(
      TemplateContentGenerator templateContentGenerator,
      @Named("eventThreadPool") Executor threadPool,
      Clock clock,
      @Named("commandProcessor") Map<String, Processor> tgCommandProcessors) {
    this.templateContentGenerator = templateContentGenerator;
    this.threadPool = threadPool;
    this.clock = clock;
    this.tgCommandProcessors = tgCommandProcessors;
  }

  @Override
  public Set<String> getTriggerKeywords() {
    return Set.of(TRIGGERING_STAGING_NAME);
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(TgChat chat, Update update) {
    return CompletableFuture.supplyAsync(
        () -> {
          final String message =
              templateContentGenerator.mergeTemplate(
                  velocityContext -> {
                    velocityContext.put(
                        "commands",
                        tgCommandProcessors.values().stream()
                            .distinct()
                            .map(
                                p ->
                                    Pair.of(
                                        p.getTriggerKeywords().stream()
                                            .sorted()
                                            .collect(Collectors.toList()),
                                        p.getLocaleDescriptionKeyword()))
                            .collect(Collectors.toList()));
                  },
                  chat.getLocale(),
                  "template/telegram/help.vm");

          final ChatContext chatContext = chat.getChatContext();
          chatContext.setStage(TRIGGERING_STAGING_NAME);
          chatContext.setStep(0);
          chatContext.setPreviousMessageUnixTimestampInSeconds(clock.now());
          chat.setChatContext(chatContext);
          chat.setHasHelpBeenShown(true);
          chat.save();

          // FIXME put all the buttons here

          return Optional.of(new SendMessage(chat.getId(), message));
        },
        threadPool);
  }
}
